﻿using Convertor.Interface;
using System;
using System.Linq;
using System.Collections;
using System.Text;
using System.Windows;
using Wpf.CommonTools.SystemExtension;
using System.Text.RegularExpressions;

namespace Convertor.Plugin.Combin
{
    public class Plugin : PluginBase, IEnumerable
    {
        #region Static Func

        static string[] LanguagePaths = new string[] { @"Convertor.Plugin.Combin;component\Assets\Lang\StringResource.zh.xaml", @"Convertor.Plugin.Combin;component\Assets\Lang\StringResource.en.xaml" };

        static Plugin()
        {
            foreach (string path in LanguagePaths)
            {
                ResourceDictionary resourceDictionary = new ResourceDictionary();
                resourceDictionary.Source = new Uri(path, UriKind.Relative);
                Application.Current.Resources.MergedDictionaries.Add(resourceDictionary);
            }
        }

        static Regex reg = new Regex(@"{\d+}");
        #endregion

        #region IPluginInfo 成员

        public override string Name
        {
            get { return UILanguage.FindResource("PluginType_Array") + " - " + UILanguage.FindResource("Plugin_Combine_Name"); }
        }

        public override string Description
        {
            get { return UILanguage.FindResource("Plugin_Combine_Desc"); }
        }

        #endregion

        #region IConvertParamters 成员

        public override IConvertParamters Output(IConvertParamters content)
        {
            IList iEnum = content.OutputContent as IList;
            if (iEnum == null)
            {
                if (Log != null) { Log(this, UILanguage.FindResource("Plugin_Common_Input_NotArray"), LogTypes.Alert); }
                return content;
            }
            string splitor = config.Splitor;

            if (config.CombineType == CombineTypes.StringFormat)
            {
                var matches = reg.Matches(splitor);
                if (Log != null) { Log(this, string.Format(UILanguage.TryFindResource("Plugin_Combin_Splitor_FormatLog"), matches.Count), LogTypes.Normal); }

                var aa = new string[iEnum.Count];
                iEnum.CopyTo(aa, 0);
                string sb;
                try
                {
                    sb = string.Format(splitor, aa);
                    if (Log != null) { Log(this, config.ToolTip, LogTypes.Normal); }
                    if (Log != null) { Log(this, string.Format(UILanguage.FindResource("Plugin_Combine_ConvertSuccess"), iEnum.Count), LogTypes.Normal | LogTypes.HidePluginName); }
                    return new ConvertParamters(content.LastInputContent, sb);
                }
                catch (Exception ex)
                {
                    if (Log != null) { Log(this, ex.ToString(), LogTypes.Alert); }
                    return new ConvertParamters(content.LastInputContent, content.OutputContent);
                }
            }
            else
            {
                StringBuilder sb = new StringBuilder();
                foreach (object o in iEnum)
                {
                    sb.Append((o ?? string.Empty).ToString());
                    sb.Append(splitor);
                }
                if (sb.Length > 0)
                {
                    sb = sb.Remove(sb.Length - config.Splitor.Length, config.Splitor.Length);
                }

                if (Log != null) { Log(this, config.ToolTip, LogTypes.Normal); }
                if (Log != null) { Log(this, string.Format(UILanguage.FindResource("Plugin_Combine_ConvertSuccess"), iEnum.Count), LogTypes.Normal | LogTypes.HidePluginName); }
                return new ConvertParamters(content.LastInputContent, sb);
            }
        }

        #endregion

        #region IPluginConfigForm 成员

        public override PluginConfigForm ConfigForm
        {
            get { return new ConfigForm(); }
        }

        #endregion

        #region IPluginConfig 成员

        private ConfigContent config;
        public override object Config
        {
            get { return config; }
            set { config = (value as ConfigContent) ?? new ConfigContent(); }
        }

        #endregion

        #region IEnumerable 成员

        IEnumerator IEnumerable.GetEnumerator()
        {
            throw new NotImplementedException();
        }

        #endregion

        #region IPluginInit 成员

        public override void InitPlugin()
        {
            ResourceDictionary resourceDictionary = new ResourceDictionary();
            resourceDictionary.Source = new Uri("Convertor.Plugin.Combin;component/Assets/Lang/StringResource.zh-CN.xaml", UriKind.Relative);
        }

        #endregion

    }
}
