﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;
using Convertor.Interface;
using Wpf.CommonTools.SystemExtension;
using System.Windows;

namespace Convertor.Plugin.Encrypt
{
    public class Plugin : PluginBase
    {
        #region Static Func

        static string[] LanguagePaths = new string[] { @"Convertor.Plugin.Encrypt;component\Assets\Lang\StringResource.zh.xaml", @"Convertor.Plugin.Encrypt;component\Assets\Lang\StringResource.en.xaml" };

        static Plugin()
        {
            foreach (string path in LanguagePaths)
            {
                ResourceDictionary resourceDictionary = new ResourceDictionary();
                resourceDictionary.Source = new Uri(path, UriKind.Relative);
                Application.Current.Resources.MergedDictionaries.Add(resourceDictionary);
            }
        }

        #endregion

        #region IPluginInfo 成员

        public override string Name
        {
            get { return UILanguage.FindResource("PluginType_Text") + " - " + UILanguage.FindResource("Plugin_Encrypt_Name"); }
        }

        public override string Description
        {
            get { return UILanguage.FindResource("Plugin_Encrypt_Desc"); }
        }


        

        

        #endregion

        #region IConvertParamters 成员

        public override IConvertParamters Output(IConvertParamters content)
        {
            if (content == null || content.OutputContent == null)
            {
                if (Log != null) { Log(this, UILanguage.FindResource("Plugin_Common_Input_Output_Null"), LogTypes.Error); }
                content.StopConvert = true;
                return content;
            }

            string value;

            switch (config.EncryptType)
            {
                case EncryptTypes.Symmetric:
                    SymmetricMethod.Key = config.Key;
                    SymmetricMethod.Encoding = config.EncodingContent.Encoder;
                    if (config.SymmetricType == DirectionTypes.Encrypt)
                        value = SymmetricMethod.Encrypto(content.OutputContent.ToString());
                    else
                        value = SymmetricMethod.Decrypto(content.OutputContent.ToString());
                    break;
                default:
                    value = ShaMethod.Encrypt(config.EncodingContent.Encoder, content.OutputContent.ToString(), config.EncryptType);
                    break;
            }

            if (Log != null) { Log(this, config.ToolTip, LogTypes.Normal); }
            return new ConvertParamters(content.LastInputContent, value);
        }

        #endregion

        #region IPluginConfigForm 成员

        public override PluginConfigForm ConfigForm
        {
            get { return new ConfigForm(); }
        }

        #endregion

        #region IPluginConfig 成员

        private ConfigContent config;
        public override object Config
        {
            get { return config; }
            set { config = (value as ConfigContent) ?? new ConfigContent(); }
        }

        #endregion

    }
}
