﻿using Convertor.Interface;
using Convertor.Plugin.Combin;
using System;
using System.IO;
using System.Windows;
using Wpf.CommonTools.SystemExtension;

namespace Convertor.Plugin.ReadFromDisk
{
    public class Plugin : PluginBase
    {
        #region Static Func

        static string[] LanguagePaths = new string[] { @"Convertor.Plugin.ReadFromDisk;component\Assets\Lang\StringResource.zh.xaml", @"Convertor.Plugin.ReadFromDisk;component\Assets\Lang\StringResource.en.xaml" };

        static Plugin()
        {
            foreach (string path in LanguagePaths)
            {
                ResourceDictionary resourceDictionary = new ResourceDictionary();
                resourceDictionary.Source = new Uri(path, UriKind.Relative);
                Application.Current.Resources.MergedDictionaries.Add(resourceDictionary);
            }
        }

        #endregion

        #region IPluginInfo 成员

        public override string Name
        {
            get { return UILanguage.FindResource("PluginType_File") + " - " + UILanguage.FindResource("Plugin_ReadFromDisk_Name"); }
        }

        public override string Description
        {
            get { return UILanguage.FindResource("Plugin_ReadFromDiske_Desc"); }
        }

        

        

        #endregion

        #region IConvertParamters 成员

        public override IConvertParamters Output(IConvertParamters content)
        {
            if (content == null || content.OutputContent == null)
            {
                if (Log != null) { Log(this, UILanguage.FindResource("Plugin_Common_Input_Output_Null"), LogTypes.Error); }
                content.StopConvert = true;
                return content;
            }

            if (Log != null) { Log(this, config.ToolTip, LogTypes.Normal); }
            string path = content.OutputContent.ToString().Trim();
            string value = string.Empty;
            try
            {
                value = File.ReadAllText(path, config.FileEncoding);
                if (Log != null) { Log(this, string.Format(UILanguage.FindResource("Plugin_ReadFromDiske_ConvertSuccess"), path), LogTypes.Normal | LogTypes.HidePluginName); }
            }
            catch (Exception ex)
            {
                if (Log != null) { Log(this, ex.Message, LogTypes.Alert); }
            }

            return new ConvertParamters(content.LastInputContent, value);
        }

        #endregion

        #region IPluginConfigForm 成员

        public override PluginConfigForm ConfigForm
        {
            get { return new ConfigForm(); }
        }

        #endregion

        #region IPluginConfig 成员

        private ConfigContent config;
        public override object Config
        {
            get { return config; }
            set { config = (value as ConfigContent) ?? new ConfigContent(); }
        }

        #endregion

    }
}
