﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;
using Convertor.Interface;
using Wpf.CommonTools.SystemExtension;

namespace Convertor.Plugin.SaveToDisk
{
    [Serializable]
    public class ConfigContent : IConfigContent, ISerializable, ICloneable, INotifyPropertyChanged
    {
        private string saveFilePath = string.Empty;

        public string SaveFilePath
        {
            get { return saveFilePath; }
            set
            {
                saveFilePath = value; if (PropertyChanged != null)
                {
                    PropertyChanged(this, new PropertyChangedEventArgs("SaveFilePath"));
                    PropertyChanged(this, new PropertyChangedEventArgs("ToolTip"));
                }
            }
        }
        private Encoding fileEncoding = Encoding.Default;
        /// <summary>
        /// 
        /// </summary>
        public Encoding FileEncoding
        {
            get { return fileEncoding ?? Encoding.Default; }
            set
            {
                fileEncoding = value; if (PropertyChanged != null)
                {
                    PropertyChanged(this, new PropertyChangedEventArgs("FileEncoding"));
                    PropertyChanged(this, new PropertyChangedEventArgs("ToolTip"));
                }
            }
        }

        public ConfigContent()
        {
        }

        public override string ToString()
        {
            return ToolTip;
        }

        #region IToolTip 成员

        public string ToolTip
        {
            get
            {
                return string.Format(UILanguage.TryFindResource("Plugin_SaveToDisk_Info"),
                    this.saveFilePath,
                    this.fileEncoding.EncodingName);
            }
        }

        #endregion

        #region ISerializable 成员

        public void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("SaveFilePath", SaveFilePath, typeof(string));
            info.AddValue("FileEncoding", FileEncoding, typeof(Encoding));
        }

        public ConfigContent(SerializationInfo info, StreamingContext context)
        {
            saveFilePath = (string)info.GetValue("SaveFilePath", typeof(string));
            fileEncoding = (Encoding)info.GetValue("FileEncoding", typeof(Encoding));
        }

        #endregion

        #region ICloneable 成员

        public object Clone()
        {
            ConfigContent a = new ConfigContent()
            {
                SaveFilePath = this.SaveFilePath,
                FileEncoding = this.FileEncoding
            };
            return a;
        }

        #endregion

        #region INotifyPropertyChanged 成员

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

    }
}
